{$IFDEF VER90}
  {$DEFINE D2}
{$ENDIF} // D2

{$IFDEF VER93}
  {$DEFINE D2}
{$ENDIF} // CPPB 1

{$IFDEF VER100}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D3

{$IFDEF VER110}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // CPPB 3

{$IFDEF VER120}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D4

{$IFDEF VER130}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D5

{$IFDEF VER140}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D6

{$IFDEF VER150}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D7

{$IFDEF VER160}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D8

{$IFDEF VER170}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D2005

{$IFDEF VER180}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D2006

{$IFDEF VER185}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D2007

{$IFDEF VER200}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D2009

{$IFDEF VER210}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D2010

{$IFDEF VER220}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D XE

{$IFDEF VER230}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D XE2

{$IFDEF VER240}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D XE3

{$IFDEF VER250}
  {$DEFINE D_XE4}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D XE4

{$IFDEF VER260}
  {$DEFINE D_XE5}
  {$DEFINE D_XE4}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D XE5

{$IFDEF VER270}
  {$DEFINE D_XE6}
  {$DEFINE D_XE5}
  {$DEFINE D_XE4}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D XE6

{$IFDEF VER280}
  {$DEFINE D_XE7}
  {$DEFINE D_XE6}
  {$DEFINE D_XE5}
  {$DEFINE D_XE4}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D XE7

{$IFDEF VER290}
  {$DEFINE D_XE8}
  {$DEFINE D_XE7}
  {$DEFINE D_XE6}
  {$DEFINE D_XE5}
  {$DEFINE D_XE4}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D XE8

{$IFDEF VER300}
  {$DEFINE D_100}
  {$DEFINE D_XE8}
  {$DEFINE D_XE7}
  {$DEFINE D_XE6}
  {$DEFINE D_XE5}
  {$DEFINE D_XE4}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D 10

{$IFDEF VER310}
  {$DEFINE D_101}
  {$DEFINE D_100}
  {$DEFINE D_XE8}
  {$DEFINE D_XE7}
  {$DEFINE D_XE6}
  {$DEFINE D_XE5}
  {$DEFINE D_XE4}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D 10.1

{$IFDEF VER320}
  {$DEFINE D_102}
  {$DEFINE D_101}
  {$DEFINE D_100}
  {$DEFINE D_XE8}
  {$DEFINE D_XE7}
  {$DEFINE D_XE6}
  {$DEFINE D_XE5}
  {$DEFINE D_XE4}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D 10.2

{$IFDEF VER330}
  {$DEFINE D_103}
  {$DEFINE D_102}
  {$DEFINE D_101}
  {$DEFINE D_100}
  {$DEFINE D_XE8}
  {$DEFINE D_XE7}
  {$DEFINE D_XE6}
  {$DEFINE D_XE5}
  {$DEFINE D_XE4}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D 10.3

{$IFDEF VER340}
  {$DEFINE D_104}
  {$DEFINE D_103}
  {$DEFINE D_102}
  {$DEFINE D_101}
  {$DEFINE D_100}
  {$DEFINE D_XE8}
  {$DEFINE D_XE7}
  {$DEFINE D_XE6}
  {$DEFINE D_XE5}
  {$DEFINE D_XE4}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D 10.4

{$IFDEF VER350}
  {$DEFINE D_110}
  {$DEFINE D_104}
  {$DEFINE D_103}
  {$DEFINE D_102}
  {$DEFINE D_101}
  {$DEFINE D_100}
  {$DEFINE D_XE8}
  {$DEFINE D_XE7}
  {$DEFINE D_XE6}
  {$DEFINE D_XE5}
  {$DEFINE D_XE4}
  {$DEFINE D_XE3}
  {$DEFINE D_XE2}
  {$DEFINE D_XE}
  {$DEFINE D2010}
  {$DEFINE D2009}
  {$DEFINE D2007}
  {$DEFINE D2006}
  {$DEFINE D2005}
  {$DEFINE D8}
  {$DEFINE D7}
  {$DEFINE D6}
  {$DEFINE D5}
  {$DEFINE D4}
  {$DEFINE D3}
  {$DEFINE D2}
{$ENDIF} // D 11.0


{$IFDEF D6}
  {$WARN SYMBOL_PLATFORM OFF} // Suppress .Net warnings
{$ENDIF}

{$IFDEF D7}
  {$WARN UNSAFE_CAST OFF} // Suppress .Net warnings
  {$WARN UNSAFE_TYPE OFF} // Suppress .Net warnings
  {$WARN UNSAFE_CODE OFF} // Suppress .Net warnings
{$ENDIF}

