{$DEFINE GTK3}
{. $DEFINE GTK3DEBUGCORE}
{. $DEFINE GTK3DEBUGSIZE}
{. $DEFINE VerboseGtk3DeviceContext}
{. $DEFINE VerboseGtk3WinApi}
{. $DEFINE GTK3DEBUGREPARENTING}
{. $DEFINE GTK3DEBUGMOUSE}
{. $DEFINE GTK3DEBUGFOCUS}
{. $DEFINE GTK3DEBUGCELLRENDERER}
{. $DEFINE GTK3DEBUGNOTIMPLEMENTED}
{. $DEFINE GTK3DEBUGKEYPRESS}
{. $DEFINE GTK3DEBUGMENUS}
{. $DEFINE GTK3DEBUGPREFERREDSIZE}



