{*
 * The MIT License (MIT)
 *
 * Copyright (c) 2020 Grupo SC10
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *}
unit lr_dotmatrix_dlg;

{$mode objfpc}{$H+}

interface

uses
  Classes,
  SysUtils,
  FileUtil,
  Forms,
  Controls,
  Graphics,
  Dialogs,
  StdCtrls,
  Spin,
  Buttons;

type

  { TlrDMDlg }

  TlrDMDlg = class(TForm)
    // buttons
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    cboOptionsLineSpacing: TComboBox;
    lblOptionsLineSpacing: TLabel;
    lblOptionsLineSpacingCustom: TLabel;
    // printer
    lblPrinter: TLabel;
    cboPrinter: TComboBox;
    // copie
    lblCopies: TLabel;
    edtCopies: TSpinEdit;
    // page
    grpOptions: TGroupBox;
    chkOptionsAutoNewPage: TCheckBox;
    lblOptionsAutoNewPageLines: TLabel;
    edtOptionsAutoNewPageLines: TSpinEdit;
    edtOptionsLineSpacingCustom: TSpinEdit;
    procedure cboOptionsLineSpacingChange(Sender: TObject);
  end;

var
  lrDMDlg: TlrDMDlg;

implementation

{$R *.lfm}

{ TlrDMDlg }

procedure TlrDMDlg.cboOptionsLineSpacingChange(Sender: TObject);
begin
  if (cboOptionsLineSpacing.ItemIndex = 0) then
  begin
    edtOptionsLineSpacingCustom.Enabled := True;
  end
  else
  begin
    edtOptionsLineSpacingCustom.Enabled := False;
    edtOptionsLineSpacingCustom.Value := 0;
  end;
end;

end.

